SystemOrganization addCategory: #'OpalCompiler-Frontend'!

CompilerContext subclass: #OCCompilerContext
	instanceVariableNames: 'parserBlock irBuilderBlock bytecodeGeneratorBlock astAnalyzerBlock mapperBlock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OpalCompiler-Frontend'!

!OCCompilerContext class methodsFor: 'defaults' stamp: ''!
default

	^(OCCompilerContext new
		parserBlock: self defaultParser;
		astAnalyzerBlock: self defaultASTAnalyzer;
		irBuilderBlock: self defaultIRBuilder;
		bytecodeGeneratorBlock: self defaultBytecodeGenerator;
		mapperBlock: self defaultMapper;
		yourself).! !

!OCCompilerContext class methodsFor: 'defaults' stamp: 'EstebanAllende 1/15/2013 11:05'!
defaultASTAnalyzer

	^[:request :methodNode | methodNode doSemanticAnalysisIn: request target]
! !

!OCCompilerContext class methodsFor: 'defaults' stamp: 'EstebanAllende 1/15/2013 11:09'!
defaultBytecodeGenerator

	^[:request :ir | ir compiledMethodWith: CompiledMethodTrailer empty].! !

!OCCompilerContext class methodsFor: 'defaults' stamp: 'EstebanAllende 1/15/2013 11:08'!
defaultIRBuilder

	^[:request :methodNode | methodNode generateIR]
			  ! !

!OCCompilerContext class methodsFor: 'defaults' stamp: ''!
defaultMapper

	^[:method :active :pc :cctx| 
		|request code kategory methodClass ast ir rir map res|
		code := method sourceCode.
		kategory := method category.
		methodClass:= method methodClass.
		request :=MethodCompileRequest new.
		request source: code;
		category: kategory;
		requestor: nil;
		trailer: (ByteArray new: 10);
		failBlock: [^self error:'Mapping failed'];
		target: methodClass.
		ast := cctx parserBlock value: request.
		cctx astAnalyzerBlock value: request value: ast.
		ir := cctx irBuilderBlock value: request value: ast.
		map := ir sourceMap.
		res := map detect:[:each| each key = pc].
		active ifTrue: [res := map before: res ifAbsent: [res]].
		res value.
	].
			  ! !

!OCCompilerContext class methodsFor: 'defaults' stamp: 'EstebanAllende 1/15/2013 11:03'!
defaultParser

	"^[:request | (PPSmalltalkParser new) 
							parse: request sourceStream contents
	 						onError: [ :msg :pos | self error: msg ]]."
						
	^[:request| RBParser
		parseMethod: request sourceStream contents
		onError: [ :msg :pos | self error: msg ]]
			  ! !

!OCCompilerContext methodsFor: 'accessing' stamp: ''!
astAnalyzerBlock

	^ astAnalyzerBlock! !

!OCCompilerContext methodsFor: 'accessing' stamp: ''!
astAnalyzerBlock: anObject

	astAnalyzerBlock := anObject! !

!OCCompilerContext methodsFor: 'accessing' stamp: ''!
bytecodeGeneratorBlock

	^ bytecodeGeneratorBlock! !

!OCCompilerContext methodsFor: 'accessing' stamp: ''!
bytecodeGeneratorBlock: anObject

	bytecodeGeneratorBlock := anObject! !

!OCCompilerContext methodsFor: 'accessing' stamp: ''!
compilerName
	^#OpalCompiler! !

!OCCompilerContext methodsFor: 'as yet unclassified' stamp: ''!
defaultStyler
	^(Smalltalk at: #OCCodeStyler) new.! !

!OCCompilerContext methodsFor: 'accessing' stamp: ''!
irBuilderBlock

	^ irBuilderBlock! !

!OCCompilerContext methodsFor: 'accessing' stamp: ''!
irBuilderBlock: anObject

	irBuilderBlock := anObject! !

!OCCompilerContext methodsFor: 'accessing' stamp: ''!
mapperBlock

	^ [:method :active :pc| mapperBlock value: method value: active value: pc value: self ]! !

!OCCompilerContext methodsFor: 'accessing' stamp: ''!
mapperBlock: anObject

	mapperBlock := anObject! !

!OCCompilerContext methodsFor: 'accessing' stamp: ''!
parserBlock

	^ parserBlock! !

!OCCompilerContext methodsFor: 'accessing' stamp: ''!
parserBlock: anObject

	parserBlock := anObject! !

!Compiler class methodsFor: '*opalcompiler-frontend-accessing' stamp: ''!
oldParserClass
	"Answer a parser class to use for parsing methods compiled by instances of the receiver."

	^Parser! !

CompilerFrontend subclass: #OCCompilerFrontend
	instanceVariableNames: 'methodCompiler'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OpalCompiler-Frontend'!

!OCCompilerFrontend class methodsFor: 'as yet unclassified' stamp: ''!
initialize
	^CompilerFrontend registerCompiler: #OpalCompiler as: self new.! !

!OCCompilerFrontend methodsFor: 'backward compatibility' stamp: 'EstebanAllende 1/15/2013 10:51'!
bcCompilerClass
	^OpalCompiler! !

!OCCompilerFrontend methodsFor: 'backward compatibility' stamp: ''!
bcDecompilerClass
	^OCFakeDecompiler ! !

!OCCompilerFrontend methodsFor: 'backward compatibility' stamp: 'EstebanAllende 1/15/2013 10:51'!
bcEvaluatorClass
	^OpalCompiler! !

!OCCompilerFrontend methodsFor: 'backward compatibility' stamp: 'EstebanAllende 1/15/2013 10:52'!
bcMethodMapper
	^DebuggerMethodMapOpal! !

!OCCompilerFrontend methodsFor: 'backward compatibility' stamp: ''!
bcParserClass
	^OCParser! !

!OCCompilerFrontend methodsFor: 'as yet unclassified' stamp: ''!
defaultCompilerContext
	
	^OCCompilerContext default.! !

!OCCompilerFrontend methodsFor: 'as yet unclassified' stamp: ''!
methodCompiler
	methodCompiler :=nil.
	^methodCompiler ifNil: [methodCompiler := OCMethodCompilerFrontend new].! !

MethodCompilerFrontend subclass: #OCMethodCompilerFrontend
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OpalCompiler-Frontend'!

!OCMethodCompilerFrontend methodsFor: 'compiling' stamp: ''!
compileWithRequest: aCompilerRequest usingCompilerContext: aCompilerContext

	| methodNode compiledMethod ir irTranslator|
	
	"Parsing"
	methodNode := aCompilerContext parserBlock value: aCompilerRequest.
	
	"Semantic Analysis"
	aCompilerContext astAnalyzerBlock value: aCompilerRequest value: methodNode.
	
	"IR generation"
	ir := aCompilerContext irBuilderBlock value: aCompilerRequest value: methodNode.
	
	"Bytecode generation"
	compiledMethod := aCompilerContext bytecodeGeneratorBlock value: aCompilerRequest value: ir.
	
	"Preparing result"
	aCompilerRequest target ifNotNil: [compiledMethod methodClass: aCompilerRequest target].
	compiledMethod selector: methodNode selector.
	^ CompiledMethodWithNode method: compiledMethod node: methodNode.! !

!OCMethodCompilerFrontend methodsFor: 'compiling' stamp: 'EstebanAllende 1/15/2013 10:52'!
evaluateWithRequest: aCompileRequest usingCompilerContext: aCompilerContext

	^OpalCompiler new evaluate: aCompileRequest source in: aCompileRequest runtimeContext to: aCompileRequest target notifying: aCompileRequest requestor ifFail: aCompileRequest failBlock logged: false.! !

!OCMethodCompilerFrontend methodsFor: 'as yet unclassified' stamp: 'EstebanAllende 7/15/2012 22:46'!
mapActiveContextToSource: aContext

	^aContext methodClass compilerContext mapperBlock value: aContext method value: true value: aContext pc.! !

!OCMethodCompilerFrontend methodsFor: 'as yet unclassified' stamp: 'EstebanAllende 7/15/2012 22:46'!
mapPasiveContextToSource: aContext

	^aContext methodClass compilerContext mapperBlock value: aContext method value: false value: aContext pc.! !

!OCMethodCompilerFrontend methodsFor: 'compiling' stamp: ''!
parseWithRequest: aCompilerRequest usingCompilerContext: aCompilerContext

	| methodNode compiledMethod ir irTranslator|
	
	"Parsing"
	methodNode := aCompilerContext parserBlock value: aCompilerRequest.
	
	^ methodNode.! !
OCCompilerFrontend initialize!
